/**
 * AutoDAO - Generic DAO on steroids implementation for Java.
 * Copyright (C) 2008  Marat Radchenko <slonopotamus@users.sourceforge.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.autodao;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.Collection;

/**
 * Helper utils.
 * NOT FOR PUBLIC USE.
 */
public final class Utils {

  /** Deny instantiation. */
  private Utils() {
  }

  /**
   * Check whether given method looks like a finder.
   * @param method method to check.
   * @return <code>true</code> if method looks like a finder.
   */
  public static boolean isFinder(final Method method) {
    return method.getName().startsWith("find");
  }

  /**
   * Returns true if given method is a single-find one.
   * @param meth method invocation.
   * @return true if single-find.
   */
  public static boolean isSingleFind(final Method meth) {
    return isFinder(meth) && !Collection.class.isAssignableFrom(meth.getReturnType());
  }

  public static Integer getLimitParameterIndex(final Method m) {
    return getAnnotatedParameterIndex(Limit.class, m);
  }

  public static Integer getOffsetParameterIndex(final Method m) {
    return getAnnotatedParameterIndex(Offset.class, m);
  }

  public static Limit getLimit(final Annotation[] arr) {
    return getInstance(arr, Limit.class);
  }

  public static Offset getOffset(final Annotation[] arr) {
    return getInstance(arr, Offset.class);
  }

  public static Named getNamed(final Annotation[] arr) {
    return getInstance(arr, Named.class);
  }

  private static <T> T getInstance(final Annotation[] arr, final Class<? extends T> klass) {
    for (final Annotation t : arr) {
      if (klass.isInstance(t)) {
        return klass.cast(t);
      }
    }
    return null;
  }

  private static Integer getAnnotatedParameterIndex(final Class<? extends Annotation> ann, final Method m) {
    for (int i = 0; i < m.getParameterAnnotations().length; i++) {
      if (getInstance(m.getParameterAnnotations()[i], ann) != null) {
        return i;
      }
    }
    return null;
  }
}
