/**
 * AutoDAO - Generic DAO on steroids implementation for Java.
 * Copyright (C) 2008  Marat Radchenko <slonopotamus@users.sourceforge.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.autodao.hibernate;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import net.sf.autodao.Dao;
import net.sf.autodao.DaoFactory;
import net.sf.autodao.PersistentEntity;
import org.hibernate.EntityMode;
import org.hibernate.Hibernate;
import org.hibernate.metadata.ClassMetadata;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.springframework.util.Assert;

/**
 * Hibernate-based implementation of {@link net.sf.autodao.DaoFactory}.
 * <p/>
 * NOT FOR PUBLIC USE.
 */
@SuppressWarnings("unchecked")
public final class HibernateDaoFactoryImpl extends HibernateDaoSupport implements DaoFactory {

  /** Cached daos. */
  private Map<Class, Dao<?, ?>> daos;

  /** Custom entity daos. */
  private Map<Class, Dao<?, ?>> customDaos = Collections.emptyMap();

  /** {@inheritDoc} */
  @Override
  public <E extends PersistentEntity<K>, K extends Serializable, D extends Dao<E, K>> D getDao(
      final Class<? extends E> klass) {
    Assert.notNull(klass, "No class given");
    final D result = (D) this.daos.get(klass);
    Assert.notNull(result, "No DAO for " + klass.getName() + " found");
    return result;
  }

  /** {@inheritDoc} */
  @Override
  public <E extends PersistentEntity<K>, K extends Serializable, D extends Dao<E, K>> D getDao(final E entity) {
    Assert.notNull(entity, "No entity given");
    return this.<E, K, D>getDao((Class<? extends E>) Hibernate.getClass(entity));
  }

  /** {@inheritDoc} */
  @Override
  protected void initDao() {
    Assert.notNull(this.customDaos);
    Assert.noNullElements(this.customDaos.values().toArray(new Object[this.customDaos.size()]));
    this.daos = new HashMap<Class, Dao<?, ?>>(this.customDaos);
    final Map<String, ClassMetadata> metas = this.getSessionFactory().getAllClassMetadata();
    for (final Map.Entry<String, ClassMetadata> en : metas.entrySet()) {
      final Class clazz = en.getValue().getMappedClass(EntityMode.POJO);
      if (!this.daos.containsKey(clazz)) {
        @SuppressWarnings("PMD.AvoidInstantiatingObjectsInLoops")
        final HibernateDaoImpl dao = new HibernateDaoImpl();
        dao.setType(clazz);
        dao.setHibernateTemplate(this.getHibernateTemplate());
        dao.afterPropertiesSet();
        this.daos.put(clazz, dao);
      }
    }
  }

  public void setCustomDaos(final Map<Class, Dao<?, ?>> customDaos) {
    this.customDaos = customDaos;
  }
}
