/**
 * AutoDAO - Generic DAO on steroids implementation for Java.
 * Copyright (C) 2008  Marat Radchenko <slonopotamus@users.sourceforge.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.autodao.hibernate;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;

import net.sf.autodao.Dao;
import net.sf.autodao.FinderExecutor;
import net.sf.autodao.PersistentEntity;
import org.hibernate.Query;
import org.hibernate.Session;
import org.springframework.beans.factory.BeanNameAware;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.orm.hibernate3.HibernateCallback;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

/**
 * Hibernate implementation of AutoDAO. This class is intentionally made final.
 * If you need custom DAO method, introduce additional bean.
 * <p/>
 * NOT FOR PUBLIC USE.
 * @param <E>
 * @param <K>
 */
@Transactional(propagation = Propagation.MANDATORY)
public final class HibernateDaoImpl<E extends PersistentEntity<K>, K extends Serializable> extends HibernateDaoSupport
    implements Dao<E, K>, FinderExecutor, BeanNameAware {

  /** Bean name. */
  private String beanName;

  /** class of persistent object. */
  private Class<E> type;

  /** {@inheritDoc} */
  @Override
  public void create(final E o) {
    this.checkObject(o);
    Assert.isTrue(!this.getHibernateTemplate().contains(o), "You're trying to persist already persistent entity: " + o);
    this.getHibernateTemplate().persist(o);
  }

  /** {@inheritDoc} */
  @Override
  public void delete(final E o) {
    this.checkObject(o);
    Assert.isTrue(this.getHibernateTemplate().contains(o), "You're trying to delete transient entity: " + o);
    this.getHibernateTemplate().delete(o);
  }

  /** {@inheritDoc} */
  @Override
  @SuppressWarnings({ "unchecked" })
  public <T> List<T> executeListFinder(final Method method, final Map<String, ?> queryArgs, final Integer limit,
                                       final Integer offset) {
    return this.getHibernateTemplate().executeFind(new HibernateCallback() {

      public Object doInHibernate(final Session session) {

        return HibernateDaoImpl.this.populateParams(HibernateDaoImpl.this.buildQuery(session, method, limit, offset),
            queryArgs).list();
      }
    });
  }

  /** {@inheritDoc} */
  @Override
  @SuppressWarnings({ "unchecked" })
  public <T> List<T> executeListFinder(final Method method, final Object[] queryArgs, final Integer limit,
                                       final Integer offset) {
    return this.getHibernateTemplate().executeFind(new HibernateCallback() {

      public Object doInHibernate(final Session session) {

        return HibernateDaoImpl.this.populateParams(HibernateDaoImpl.this.buildQuery(session, method, limit, offset),
            queryArgs).list();
      }
    });
  }

  /** {@inheritDoc} */
  @Override
  @SuppressWarnings({ "unchecked" })
  public <T> T executeSingleFinder(final Method method, final Map<String, ?> queryArgs) {
    return (T) this.getHibernateTemplate().execute(new HibernateCallback() {

      public Object doInHibernate(final Session session) {

        return HibernateDaoImpl.this.populateParams(HibernateDaoImpl.this.buildQuery(session, method, null, null),
            queryArgs).uniqueResult();
      }
    });
  }

  /** {@inheritDoc} */
  @Override
  @SuppressWarnings({ "unchecked" })
  public <T> T executeSingleFinder(final Method method, final Object[] queryArgs) {
    return (T) this.getHibernateTemplate().execute(new HibernateCallback() {

      public Object doInHibernate(final Session session) {

        return HibernateDaoImpl.this.populateParams(HibernateDaoImpl.this.buildQuery(session, method, null, null),
            queryArgs).uniqueResult();
      }
    });
  }

  /** {@inheritDoc} */
  @Override
  @SuppressWarnings({ "unchecked" })
  public E get(final K id) {
    Assert.notNull(id, "Id cannot be null");
    return (E) this.getHibernateTemplate().get(this.type, id);
  }

  /** {@inheritDoc} */
  @Override
  @SuppressWarnings({ "unchecked" })
  public E merge(final E o) {
    return (E) this.getHibernateTemplate().merge(o);
  }

  /** {@inheritDoc} */
  @Override
  public void refresh(final E o) {
    this.getHibernateTemplate().refresh(o);
  }

  /** {@inheritDoc} */
  @Override
  public void saveOrUpdate(final E o) {
    this.checkObject(o);
    this.getHibernateTemplate().saveOrUpdate(o);
  }

  /** {@inheritDoc} */
  @Override
  public void setBeanName(final String name) {
    this.beanName = name;
  }

  @Required
  public void setType(final Class<E> type) {
    this.type = type;
  }

  /** {@inheritDoc} */
  @Override
  public String toString() {
    return "AutoDao [" + this.type.getName() + ']';
  }

  /** {@inheritDoc} */
  @Override
  public void update(final E o) {
    this.checkObject(o);
    Assert.isTrue(this.getHibernateTemplate().contains(o), "You're trying to update transient entity: " + o);
    this.getHibernateTemplate().update(o);
  }

  /** {@inheritDoc} */
  @Override
  protected void initDao() {
    Assert.notNull(this.type, "You forgot to call #setType() on  bean '" + this.beanName + '\'');
  }

  private Query buildQuery(final Session s, final Method m, final Integer limit, final Integer offset) {
    final Query q = s.getNamedQuery(HibernateUtils.getNamedQueryName(s.getSessionFactory(), this.type, m));
    if (limit != null) {
      q.setMaxResults(limit);
    }
    if (offset != null) {
      q.setFirstResult(offset);
    }
    return q;
  }

  /**
   * Check entity.
   * @param o entity.
   */
  private void checkObject(final E o) {
    Assert.notNull(o, "Entity cannot be null");
    Assert.isAssignable(this.type, o.getClass(), "Invalid entity type");
  }

  private Query populateParams(final Query q, final Map<String, ?> params) {
    for (final Map.Entry<String, ?> o : params.entrySet()) {
      q.setParameter(o.getKey(), o.getValue());
    }
    return q;
  }

  private Query populateParams(final Query q, final Object[] params) {
    int i = 0;
    for (final Object o : params) {
      q.setParameter(i, o);
      i++;
    }
    return q;
  }
}
